model BouncingHead
  SlidingMass head(m=1.2);
  SlidingMass upperBody(m=14);
  SlidingMass arms(m=3.2);
  SlidingMass lowerBody(m=24);
  GvtForce headGvt(m=head.m);
  GvtForce upperBodyGvt(m=upperBody.m);
  GvtForce armsGvt(m=arms.m);
  GvtForce lowerBodyGvt(m=lowerBody.m);
  SpringDamper body_head(s_rel0=0.1, c=0.3, d=0.1*9.8) "SpringDamper connecting head and upperbody";
  SpringDamper body_arms(s_rel0=0.05, c=8, d=10*9.8) "SpringDamper connecting upperbody and arms";
  SpringDamper upper_lower_body(s_rel0=0.05, c=3, d=12*9.8) "SpringDamper connecting upperbody and lowerbody";
  SpringDamper fixed_lowerBody(s_rel0=0.05, c=1, d=10*9.8) "SpringDamper connecting lowerbody and fixed point";
  SineForce f "external input force";
  Fixed fixedPoint;
equation 
  connect(headGvt.flange_b, head.flange_a);
  connect(upperBodyGvt.flange_b, upperBody.flange_a);
  connect(armsGvt.flange_b, arms.flange_a);
  connect(lowerBodyGvt.flange_b, lowerBody.flange_a);
  connect(f.flange_b, lowerBody.flange_b);
  connect(body_head.flange_a, head.flange_b);
  connect(body_head.flange_b, upperBody.flange_a);
  connect(body_arms.flange_a, upperBody.flange_b);
  connect(upperBody.flange_b, upper_lower_body.flange_a);
  connect(body_arms.flange_b, arms.flange_a);
  connect(upper_lower_body.flange_b, lowerBody.flange_a);
  connect(fixedPoint.flange_b, fixed_lowerBody.flange_b);
  connect(lowerBody.flange_b, fixed_lowerBody.flange_a);
end BouncingHead;

model SpringDamper "Linear 1D translational spring and damper in parallel"
  extends Compliant;
  parameter Real s_rel0=0 "unstretched spring length";
  parameter Real c=1 "spring constant";
  parameter Real d=1 "damping constant";
  Real v_rel "relative velocity between flange_a and flange_b";
equation 
  v_rel=der(s_rel);
  f=c*(s_rel-s_rel0)+d*v_rel;
end SpringDamper;

model SlidingMass "Sliding mass with inertia"
  extends Rigid;
  parameter Real m=1 "mass of the sliding mass";
  Real v "absolute velocity of component";
  Real a "absolute acceleration of component";
equation 
  v=der(s);
  a=der(v);
  m*a=flange_a.f+flange_b.f;
end SlidingMass;

model SineForce "External force acting on a sliding mass"
  constant Real pi=3.1416;
  parameter Real p_freqHz=1;
  Real f "driving force";
  Flange_b flange_b;
equation 
  f=sin(2*pi*p_freqHz*time);
  flange_b.f=(-f);
end SineForce;

model GvtForce "Gravity force acting on a sliding mass"
  parameter Real m=1;
  Real f "driving force";
  Flange_b flange_b;
equation 
  f=m*9.8;
  flange_b.f=(-f);
end GvtForce;

model Fixed "fixed flange"
  parameter Real s0=0;
  Flange_b flange_b;
equation 
  flange_b.s=s0;
end Fixed;

partial model Compliant "Compliant connection of two translational 1D flanges"
  Flange_a flange_a "(left) driving flange (flange axis directed INTO cut plane, e. g. from left to right)";
  Flange_b flange_b "(right) driven flange (flange axis directed OUT OF cut plane)";
  Real s_rel "relative distance (= flange_b.s - flange_a.s)";
  Real f "forcee between flanges (positive in direction of flange axis R)";
equation 
  s_rel=flange_b.s-flange_a.s;
  flange_b.f=f;
  flange_a.f=(-f);
end Compliant;

partial model Rigid "Rigid connection of two translational 1D flanges "
  Real s "absolute position of center of component (s = flange_a.s + L/2 = flange_b.s - L/2)";
  parameter Real L=0 "length of component from left flange to right flange (= flange_b.s - flange_a.s)";
  Flange_a flange_a "(left) driving flange (flange axis directed INTO cut plane, i. e. from left to right)";
  Flange_b flange_b "(right) driven flange (flange axis directed OUT OF cut plane, i. e. from right to left)";
equation 
  flange_a.s=s-L/2;
  flange_b.s=s+L/2;
end Rigid;

connector Flange_b "right 1D translational flange (flange axis directed OUT OF cut plane)"
  Real s "absolute position of flange";
  flow Real f "cut force directed into flange";
end Flange_b;

connector Flange_a "(left) 1D translational flange (flange axis directed INTO cut plane, e. g. from left to right)"
  Real s "absolute position of flange";
  flow Real f "cut force directed into flange";
end Flange_a;

